/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.archivierung;

import java.util.function.Consumer;
import java.util.function.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.event.EventListener;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.stereotype.Component;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.archive.common.callcontext.CallContextUser;
import de.ozgcloud.archive.common.callcontext.CurrentUserService;
import de.ozgcloud.archive.common.command.CommandService;
import de.ozgcloud.archive.common.errorhandling.TimeoutException;
import de.ozgcloud.archive.vorgang.VorgangService;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.command.VorgangLockedEvent;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component
@RequiredArgsConstructor
@Log4j2
public class ArchiveEventListener {

	static final int MAXIMUM_CHECKS_FOR_PENDING_COMMANDS = 3;
	static final int WAIT_INTERVAL = 30 * 1000;
	private static final String LOG_MESSAGE_TEMPLATE = "{}. Command failed.";
	private static final String ERROR_MESSAGE_TEMPLATE = "Error on executing %s Command (id: %s).";

	static final String ARCHIVE_VORGANG_ORDER = "ARCHIVE_VORGANG";
	private static final String IS_ARCHIVE_VORGANG_EVENT = "{T(de.ozgcloud.archive.archivierung.ArchiveEventListener).IS_ARCHIVE_VORGANG_COMMAND.test(event.getSource())}";
	public static final Predicate<Command> IS_ARCHIVE_VORGANG_COMMAND = command -> ARCHIVE_VORGANG_ORDER.equals(command.getOrder());

	private static final String IS_LOCKED_BY_ARCHIVE_MANAGER_EVENT = "{T(de.ozgcloud.archive.archivierung.ArchiveEventListener)."
			+ "IS_LOCK_BY_ARCHIVE_MANAGER_COMMAND.test(event.getCommand())}";
	public static final Predicate<Command> IS_LOCK_BY_ARCHIVE_MANAGER_COMMAND = command -> CallContextUser.ARCHIVE_MANAGER_CLIENT_NAME
			.equals(command.getCreatedByClientName());

	@Qualifier(ArchiveManagerConfiguration.CURRENT_USER_SERVICE_NAME)
	private final CurrentUserService currentUserService;

	@Qualifier(ArchiveManagerConfiguration.VORGANG_SERVICE_NAME)
	private final VorgangService vorgangService;

	private final ApplicationEventPublisher eventPublisher;

	@Qualifier(ArchiveManagerConfiguration.COMMAND_SERVICE_NAME)
	private final CommandService commandService;
	private final ArchiveService archiveService;

	@EventListener(condition = IS_ARCHIVE_VORGANG_EVENT)
	void onArchiveVorgangEvent(CommandCreatedEvent event) {
		runWithSecurityContext(event.getSource(), this::doLockVorgang);
	}

	void doLockVorgang(Command command) {
		archiveService.setVorgangArchiving(command.getVorgangId());
		vorgangService.lockVorgang(command);
	}

	@EventListener(condition = IS_LOCKED_BY_ARCHIVE_MANAGER_EVENT)
	public void onVorgangLockedEvent(VorgangLockedEvent event) {
		waitForPendingCommandsToFinish(event.getCommand().getVorgangId(), WAIT_INTERVAL);
		runWithSecurityContext(event.getCommand(), archiveService::archiveVorgang);
	}

	void waitForPendingCommandsToFinish(String vorgangId, long waitIntervalInMillis) {
		var numberOfAttempts = 0;
		while (commandService.hasPendingCommandsExceptWithOrder(vorgangId, ARCHIVE_VORGANG_ORDER)) {
			numberOfAttempts++;
			if (numberOfAttempts >= MAXIMUM_CHECKS_FOR_PENDING_COMMANDS) {
				throw new TimeoutException("Waiting for pending commands");
			}
			try {
				Thread.sleep(waitIntervalInMillis);
			} catch (InterruptedException e) {
				LOG.error("Error while waiting for commands to finish.", e);
				Thread.currentThread().interrupt();
			}
		}
	}

	void runWithSecurityContext(Command command, Consumer<Command> commandExecutor) {
		SecurityContext prevContext = null;
		try {
			prevContext = currentUserService.startAndReturnPreviousSecurityContext(command);
			commandExecutor.accept(command);
		} catch (Exception e) {
			var errorMessage = ERROR_MESSAGE_TEMPLATE.formatted(command.getOrder(), command.getId());
			LOG.error(LOG_MESSAGE_TEMPLATE, errorMessage, e);
			eventPublisher.publishEvent(new CommandFailedEvent(command.getId(), buildErrorMessage(errorMessage, e)));
		} finally {
			currentUserService.resetSecurityContext(prevContext);
		}
	}

	private String buildErrorMessage(String message, Exception cause) {
		try {
			StringBuilder sb = new StringBuilder(message);
			if (StringUtils.isNotBlank(cause.getMessage())) {
				sb.append(" Cause: ").append(cause.getMessage());
			}
			return sb.toString();
		} catch (Exception e2) {
			LOG.error("Error in building Error Message (sick).", e2);
			return message;
		}
	}
}
