/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common.callcontext;

import java.util.Objects;
import java.util.Optional;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import de.ozgcloud.archive.ArchiveManagerConfiguration;
import de.ozgcloud.command.Command;
import de.ozgcloud.common.grpc.GrpcUtil;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service(ArchiveManagerConfiguration.CURRENT_USER_SERVICE_NAME)
public class CurrentUserService {

	private final AuthenticationTrustResolver trustResolver;

	public CallContextUser getUser() {
		return findUser().orElseThrow(() -> new IllegalStateException("No authenticated User found"));
	}

	public Optional<CallContextUser> findUser() {
		return findTrustedAuthentication()
				.map(this::mapToCallContextUser)
				// remove this filter as soon we have a trusted authentication
				.filter(user -> StringUtils.isNotBlank(user.getClientName()));
	}

	CallContextUser mapToCallContextUser(Authentication auth) {
		return auth instanceof CallContextAuthenticationToken token ? token.getUser() : buildDefaultCallContextUser(auth);
	}

	CallContextUser buildDefaultCallContextUser(Authentication auth) {
		var defaultContextUser = CallContextUser.builder().clientName(auth.getName());
		auth.getAuthorities().stream().map(GrantedAuthority::getAuthority).forEach(defaultContextUser::accessLimitedToOrgaId);
		return defaultContextUser.build();
	}

	public Optional<Authentication> findAuthentication() {
		return findTrustedAuthentication();
	}

	Optional<Authentication> findTrustedAuthentication() {
		return Optional.ofNullable(SecurityContextHolder.getContext())
				.map(SecurityContext::getAuthentication)
				.filter(auth -> !trustResolver.isAnonymous(auth))
				.filter(Authentication::isAuthenticated);
	}

	public SecurityContext startAndReturnPreviousSecurityContext(Command command) {
		return startAndReturnPreviousSecurityContext(createUser(command));
	}

	CallContextUser createUser(Command command) {
		return CallContextUser.builder()
				.userId(Optional.ofNullable(command.getCreatedBy()))
				.userName(Optional.ofNullable(command.getCreatedByName()))
				.clientName(command.getCreatedByClientName())
				.requestId(getRequestId())
				.accessLimited(false)
				.build();
	}

	String getRequestId() {
		return ThreadContext.getContext().getOrDefault(GrpcUtil.KEY_REQUEST_ID, UUID.randomUUID().toString());
	}

	public SecurityContext startAndReturnPreviousSecurityContext(CallContextUser user) {
		var prevContext = SecurityContextHolder.getContext();

		SecurityContextHolder.clearContext();
		SecurityContextHolder.getContext().setAuthentication(CallContextAuthenticationToken.authenticate(user));

		return prevContext;
	}

	public void resetSecurityContext(SecurityContext context) {
		SecurityContextHolder.clearContext();
		if (Objects.nonNull(context)) {
			SecurityContextHolder.setContext(context);
		}
	}
}