/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.archivierung;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.function.Consumer;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.security.core.context.SecurityContext;

import de.ozgcloud.archive.common.callcontext.CurrentUserService;
import de.ozgcloud.archive.common.command.CommandService;
import de.ozgcloud.archive.common.errorhandling.TimeoutException;
import de.ozgcloud.archive.vorgang.VorgangService;
import de.ozgcloud.archive.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEventTestFactory;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.command.VorgangLockedEvent;

class ArchiveEventListenerTest {

	private static final Command COMMAND = CommandTestFactory.create();

	@Spy
	@InjectMocks
	private ArchiveEventListener eventListener;

	@Mock
	private CurrentUserService currentUserService;
	@Mock
	private VorgangService vorgangService;
	@Mock
	private ApplicationEventPublisher eventPublisher;
	@Mock
	private CommandService commandService;
	@Mock
	private ArchiveService archiveService;

	@Captor
	private ArgumentCaptor<Consumer<Command>> commandExecutorCapture;

	@Nested
	class TestOnArchiveEvent {

		@BeforeEach
		void init() {
			doNothing().when(eventListener).doLockVorgang(any());
		}

		@Test
		void shouldCallRunWithSecurityContext() {
			doNothing().when(eventListener).runWithSecurityContext(any(), any());

			eventListener.onArchiveVorgangEvent(CommandCreatedEventTestFactory.withCommand(COMMAND));

			verify(eventListener).runWithSecurityContext(eq(COMMAND), commandExecutorCapture.capture());
			commandExecutorCapture.getValue().accept(COMMAND);
			verify(eventListener).doLockVorgang(COMMAND);
		}
	}

	@Nested
	class TestDoLockVorgang {

		@Test
		void shouldCallSetVorgangArchiving() {
			doLockVorgang();

			verify(archiveService).setVorgangArchiving(CommandTestFactory.VORGANG_ID);
		}

		@Test
		void shouldCallLockVorgang() {
			doLockVorgang();

			verify(vorgangService).lockVorgang(COMMAND);
		}

		private void doLockVorgang() {
			eventListener.doLockVorgang(COMMAND);
		}
	}

	@Nested
	class TestRunWithSecurityContext {

		@Mock
		private Consumer<Command> commandExecutor;
		@Mock
		private SecurityContext secContext;
		@Captor
		private ArgumentCaptor<CommandFailedEvent> commandFailedEventCaptor;

		@BeforeEach
		void init() {
			when(currentUserService.startAndReturnPreviousSecurityContext(any(Command.class))).thenReturn(secContext);
		}

		@Test
		void shouldStartSecurityContext() {
			eventListener.runWithSecurityContext(COMMAND, commandExecutor);

			verify(currentUserService).startAndReturnPreviousSecurityContext(COMMAND);
		}

		@Test
		void shouldExecuteCommand() {
			eventListener.runWithSecurityContext(COMMAND, commandExecutor);

			verify(commandExecutor).accept(COMMAND);
		}

		@Test
		void shouldResetSecurityContext() {
			eventListener.runWithSecurityContext(COMMAND, commandExecutor);

			verify(currentUserService).resetSecurityContext(secContext);
		}

		@Test
		void shouldResetSecurityContextAfterException() {
			doThrow(new RuntimeException("ups")).when(commandExecutor).accept(any());

			eventListener.runWithSecurityContext(COMMAND, commandExecutor);

			verify(currentUserService).resetSecurityContext(secContext);
		}

		@Test
		void shouldPublishCommandFailedEvent() {
			doThrow(new RuntimeException("ups")).when(commandExecutor).accept(any());

			eventListener.runWithSecurityContext(COMMAND, commandExecutor);

			verify(eventPublisher).publishEvent(commandFailedEventCaptor.capture());
			assertThat(commandFailedEventCaptor.getValue().getSource()).isEqualTo(CommandTestFactory.ID);
			assertThat(commandFailedEventCaptor.getValue().getErrorMessage()).isNotEmpty();
		}
	}

	@Nested
	class TestOnVorgangLockedEvent {

		private final Command command = CommandTestFactory.create();
		private final VorgangLockedEvent event = new VorgangLockedEvent(command);

		@BeforeEach
		void mock() {
			doNothing().when(eventListener).waitForPendingCommandsToFinish(any(), anyLong());
		}

		@Test
		void shouldCallWaitForPendingCommandsToFinish() {
			onVorgangLockedEvent();

			verify(eventListener).waitForPendingCommandsToFinish(CommandTestFactory.VORGANG_ID, ArchiveEventListener.WAIT_INTERVAL);
		}

		@Test
		void shouldCallRunWithSecurityContext() {
			onVorgangLockedEvent();

			verify(eventListener).runWithSecurityContext(eq(command), any());
		}

		@Nested
		class TestCommandExecuter {

			@BeforeEach
			void mock() {
				doNothing().when(eventListener).runWithSecurityContext(any(), any());
			}

			@Test
			void shouldCallRunWithSecurityContext() {
				onVorgangLockedEvent();

				verify(eventListener).runWithSecurityContext(eq(command), commandExecutorCapture.capture());
				commandExecutorCapture.getValue().accept(command);

				verify(archiveService).archiveVorgang(command);

			}
		}

		private void onVorgangLockedEvent() {
			eventListener.onVorgangLockedEvent(event);
		}
	}

	@Nested
	class TestWaitForPendingCommandsToFinish {

		@Test
		void shouldCallCommandServiceForPendingCommandsOnce() {
			when(commandService.hasPendingCommandsExceptWithOrder(any(), any())).thenReturn(false);

			waitForPendingCommandsToFinish();

			verify(commandService).hasPendingCommandsExceptWithOrder(VorgangWithEingangTestFactory.ID, ArchiveEventListener.ARCHIVE_VORGANG_ORDER);
		}

		@Test
		void shouldCallCommandServiceForPendingCommandsTwice() {
			when(commandService.hasPendingCommandsExceptWithOrder(any(), any())).thenReturn(true)
					.thenReturn(false);

			waitForPendingCommandsToFinish();

			verify(commandService, times(2)).hasPendingCommandsExceptWithOrder(VorgangWithEingangTestFactory.ID,
					ArchiveEventListener.ARCHIVE_VORGANG_ORDER);
		}

		@Test
		void shouldCallAbortWaitingAfterMaxAttempts() {
			when(commandService.hasPendingCommandsExceptWithOrder(any(), any())).thenReturn(true);

			var timeoutExceptionThrown = waitForPendingCommandsToFinishAndCatchException();

			verify(commandService, times(ArchiveEventListener.MAXIMUM_CHECKS_FOR_PENDING_COMMANDS))
					.hasPendingCommandsExceptWithOrder(VorgangWithEingangTestFactory.ID, ArchiveEventListener.ARCHIVE_VORGANG_ORDER);
			assertThat(timeoutExceptionThrown).isTrue();
		}

		private boolean waitForPendingCommandsToFinishAndCatchException() {
			try {
				waitForPendingCommandsToFinish();
			} catch (TimeoutException e) {
				return true;
			}
			return false;
		}

		private void waitForPendingCommandsToFinish() {
			eventListener.waitForPendingCommandsToFinish(VorgangWithEingangTestFactory.ID, 1);
		}
	}

}