/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.archive.export.IdentifikationObjektTypeTestFactory;
import de.ozgcloud.archive.file.OzgFile;
import de.ozgcloud.archive.file.OzgFileTestFactory;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.VersionType;

class AnlageDokumentTypeBuilderTest {

	private final OzgFile ozgFile = OzgFileTestFactory.create();
	private static final ZonedDateTime CREATED_AT = ZonedDateTime.now();
	@Spy
	private AnlageDokumentTypeBuilder builder = AnlageDokumentTypeBuilder.builder().withOzgFile(ozgFile).withCreatedAt(CREATED_AT);

	@Nested
	class TestBuild {
		private MockedStatic<IdentifikationObjektTypeBuilder> identifikationObjektTypeBuilderMockedStatic;
		@Mock
		private IdentifikationObjektTypeBuilder identifikationObjektTypeBuilder;
		private final IdentifikationObjektType identifikationObjektType = IdentifikationObjektTypeTestFactory.create();

		private MockedStatic<VersionTypeBuilder> versionTypeBuilderMockedStatic;
		@Mock
		private VersionTypeBuilder versionTypeBuilder;
		private final VersionType version = VersionTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			identifikationObjektTypeBuilderMockedStatic = mockStatic(IdentifikationObjektTypeBuilder.class);
			identifikationObjektTypeBuilderMockedStatic.when(IdentifikationObjektTypeBuilder::builder).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withObjectID(OzgFileTestFactory.ID.toString())).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.build()).thenReturn(identifikationObjektType);

			versionTypeBuilderMockedStatic = mockStatic(VersionTypeBuilder.class);
			versionTypeBuilderMockedStatic.when(VersionTypeBuilder::builder).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withCreatedAt(CREATED_AT)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withOzgFile(ozgFile)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.build()).thenReturn(version);
		}

		@AfterEach
		void tearDown() {
			identifikationObjektTypeBuilderMockedStatic.close();
			versionTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetObjectId() {
			builder.build();

			verify(identifikationObjektTypeBuilder).withObjectID(ozgFile.getId().toString());
		}

		@Test
		void shouldBuildIdentifikationObjectType() {
			builder.build();

			verify(identifikationObjektTypeBuilder).build();
		}

		@Test
		void shouldHaveIdentifikation() {
			var anlage = builder.build();

			assertThat(anlage.getIdentifikation()).isEqualTo(identifikationObjektType);
		}

		@Test
		void shouldSetCreatedAt() {
			builder.build();

			verify(versionTypeBuilder).withCreatedAt(CREATED_AT);
		}

		@Test
		void shouldSetOzgFile() {
			builder.build();

			verify(versionTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldBuildVersionType() {
			builder.build();

			verify(versionTypeBuilder).build();
		}

		@Test
		void shouldHaveVersion() {
			var anlage = builder.build();

			assertThat(anlage.getVersion()).contains(version);
		}
	}
}
