/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.historie;

import static de.ozgcloud.archive.historie.ChangeHistoryBuilderTest.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandTestFactory;
import de.ozgcloud.archive.common.command.CommandWithPrevious;
import de.ozgcloud.archive.common.command.CommandWithPreviousTestFactory;

public class StatusChangeHistoryBuilderTest {

	private static final String ORGANISATIONSEINHEITEN_ID = UUID.randomUUID().toString();

	private final ArchiveManagerCommand previousCommand = commandFinishedAt(LocalDateTime.of(2023, 5, 1, 12, 0));
	private final ArchiveManagerCommand command = commandFinishedAt(LocalDateTime.of(2023, 6, 1, 12, 0));
	private final List<ArchiveManagerCommand> commands = List.of(previousCommand, command);

	@Spy
	private StatusChangeHistoryBuilder builder = StatusChangeHistoryBuilder.builder()
			.withCommands(commands)
			.withOrganisationseinheitenID(ORGANISATIONSEINHEITEN_ID);

	@Nested
	class TestIsRelevant {

		@ValueSource(strings = { "VORGANG_ANNEHMEN", "VORGANG_VERWERFEN", "VORGANG_ZURUECKHOLEN", "VORGANG_BEARBEITEN",
				"VORGANG_BESCHEIDEN", "VORGANG_ABSCHLIESSEN", "VORGANG_ZUM_LOESCHEN_MARKIEREN", "VORGANG_LOESCHEN", "VORGANG_ZURUECKSTELLEN",
				"VORGANG_WIEDEREROEFFNEN" })
		@ParameterizedTest
		void shouldReturnTrue(String order) {
			var isStatusChangeCommand = builder.isRelevant(CommandTestFactory.createBuilder().order(order).build());

			assertThat(isStatusChangeCommand).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var order = LoremIpsum.getInstance().getWords(1);

			var isStatusChangeCommand = builder.isRelevant(CommandTestFactory.createBuilder().order(order).build());

			assertThat(isStatusChangeCommand).isFalse();
		}
	}

	@Nested
	class TestToCommandWithChangeValues {

		private static final String EXPECTED_STATUS_BEFORE_CHANGE = LoremIpsum.getInstance().getWords(2);
		private static final String EXPECTED_STATUS_AFTER_CHANGE = LoremIpsum.getInstance().getWords(2);

		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command)
				.previous(previousCommand).build();

		@BeforeEach
		void init() {
			doReturn(EXPECTED_STATUS_BEFORE_CHANGE).when(builder).getStatusBeforeChange(commandWithPrevious);
			doReturn(EXPECTED_STATUS_AFTER_CHANGE).when(builder).getStatusAfterChange(commandWithPrevious);
		}

		@Test
		void shouldSetCommand() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.command()).isEqualTo(command);
		}

		@Test
		void shouldSetValueBeforeChange() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.valueBeforeChange()).isEqualTo(EXPECTED_STATUS_BEFORE_CHANGE);
		}

		@Test
		void shouldSetValueAfterChange() {
			var commandWithChangeValues = callBuilder();

			assertThat(commandWithChangeValues.valueAfterChange()).isEqualTo(EXPECTED_STATUS_AFTER_CHANGE);
		}

		private ChangeHistoryBuilder.CommandWithChangeValues callBuilder() {
			return builder.toCommandWithChangeValues(commandWithPrevious);
		}
	}

	@Nested
	class TestGetStatusBeforeChange {

		private final CommandWithPrevious commandWithoutPrevious = CommandWithPreviousTestFactory.createBuilder().command(previousCommand)
				.previous(null).build();
		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command)
				.previous(previousCommand).build();

		@Test
		void shouldReturnStatusBeforeChange() {
			var status = "Status";
			doReturn(status).when(builder).getStatus(previousCommand);

			var statusBeforeChange = builder.getStatusBeforeChange(commandWithPrevious);

			assertThat(statusBeforeChange).isEqualTo(status);
		}

		@Test
		void shouldReturnDefaultValue() {
			var status = builder.getStatusBeforeChange(commandWithoutPrevious);

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.NEU);
		}
	}

	@Nested
	class TestGetStatusAfterChange {

		private final CommandWithPrevious commandWithPrevious = CommandWithPreviousTestFactory.createBuilder().command(command)
				.previous(previousCommand).build();

		@Test
		void shouldReturnStatusAfterChange() {
			var status = "Status";
			doReturn(status).when(builder).getStatus(command);

			var statusAfterChange = builder.getStatusAfterChange(commandWithPrevious);

			assertThat(statusAfterChange).isEqualTo(status);
		}
	}

	@Nested
	class TestGetStatus {

		@ParameterizedTest
		@ValueSource(strings = { "VORGANG_ANNEHMEN", "VORGANG_ZURUECKSTELLEN" })
		void shouldReturnAngenommen(String order) {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order(order).build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.ANGENOMMEN);
		}

		@Test
		void shouldReturnVerworfen() {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order("VORGANG_VERWERFEN").build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.VERWORFEN);
		}

		@Test
		void shouldReturnNeu() {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order("VORGANG_ZURUECKHOLEN").build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.NEU);
		}

		@ParameterizedTest
		@ValueSource(strings = { "VORGANG_BEARBEITEN", "VORGANG_WIEDEREROEFFNEN" })
		void shouldReturnInBearbeitung(String order) {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order(order).build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.IN_BEARBEITUNG);
		}

		@Test
		void shouldReturnBeschieden() {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order("VORGANG_BESCHEIDEN").build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.BESCHIEDEN);
		}

		@Test
		void shouldReturnAbschliessen() {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order("VORGANG_ABSCHLIESSEN").build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.ABGESCHLOSSEN);
		}

		@Test
		void shouldReturnZuLoeschen() {
			var status = builder
					.getStatus(CommandTestFactory.createBuilder().order("VORGANG_ZUM_LOESCHEN_MARKIEREN").build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.ZU_LOESCHEN);
		}

		@Test
		void shouldReturnGeloescht() {
			var status = builder.getStatus(CommandTestFactory.createBuilder().order("VORGANG_LOESCHEN").build());

			assertThat(status).isEqualTo(StatusChangeHistoryBuilder.GELOESCHT);
		}

		@Test
		void shouldtThrowIllegalArgumentException() {
			var command = CommandTestFactory.createBuilder().order(LoremIpsum.getInstance().getWords(1)).build();

			assertThatThrownBy(() -> builder.getStatus(command))
					.isInstanceOf(IllegalArgumentException.class);
		}
	}
}
