/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.historie;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;
import java.util.function.Function;

import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.archive.common.user.UserProfile;
import de.ozgcloud.archive.common.user.UserProfileTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;

class UserProfileCacheTest {

	private static final String USER_ID = UUID.randomUUID().toString();
	private static final UserProfile EXPECTED_USER_PROFILE = UserProfileTestFactory.create();

	@Mock
	private Function<String, Optional<UserProfile>> findUserProfileForUserId;
	@InjectMocks
	private UserProfileCache cache;

	@Test
	void shouldSearchForUserProfileThatWasNotCached() {
		givenFindUserProfileForUserIdReturnsUserProfile();

		cache.getUserProfile(USER_ID);

		verify(findUserProfileForUserId).apply(USER_ID);
	}

	@Test
	void shouldReturnFoundUserProfile() {
		givenFindUserProfileForUserIdReturnsUserProfile();

		var userProfile = cache.getUserProfile(USER_ID);

		assertThat(userProfile).contains(EXPECTED_USER_PROFILE);
	}

	@Test
	void shouldAddFoundUserProfileToCache() {
		givenFindUserProfileForUserIdReturnsUserProfile();

		cache.getUserProfile(USER_ID);

		assertThat(getUserProfileFromCache()).contains(EXPECTED_USER_PROFILE);
	}

	@Test
	void shouldNotSearchForCachedUserProfile() {
		givenUserProfileIsInCache();

		cache.getUserProfile(USER_ID);

		verifyNoInteractions(findUserProfileForUserId);
	}

	@Test
	void shouldReturnUserProfileFromCache() {
		givenUserProfileIsInCache();

		var userProfile = cache.getUserProfile(USER_ID);

		assertThat(userProfile).contains(EXPECTED_USER_PROFILE);
	}

	@Test
	void shouldThrowExceptionIfUserProfileCannotBeFound() {
		when(findUserProfileForUserId.apply(USER_ID)).thenReturn(null);

		assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> cache.getUserProfile(USER_ID))
				.withMessageStartingWith("UserProfile not found for user with ID " + USER_ID);
	}

	private void givenFindUserProfileForUserIdReturnsUserProfile() {
		when(findUserProfileForUserId.apply(USER_ID)).thenReturn(Optional.of(EXPECTED_USER_PROFILE));
	}

	private void givenUserProfileIsInCache() {
		var cache = new HashMap<String, Optional<UserProfile>>();
		cache.put(USER_ID, Optional.of(EXPECTED_USER_PROFILE));
		ReflectionTestUtils.setField(this.cache, "cache", cache);
	}

	@SuppressWarnings("unchecked")
	private Optional<UserProfile> getUserProfileFromCache() {
		return ((Map<String, Optional<UserProfile>>) Objects.requireNonNull(ReflectionTestUtils.getField(cache, "cache"))).get(USER_ID);
	}
}
