/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.archive.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedConstruction;
import org.mockito.Spy;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.archive.common.callcontext.CallContextProvider;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;

class KommentarRemoteServiceTest {

	@Spy
	@InjectMocks
	private KommentarRemoteService service;
	@Mock
	private VorgangAttachedItemServiceBlockingStub grpcService;
	@Mock
	private KommentarMapper mapper;
	@Mock
	private CallContextProvider contextProvider;

	@DisplayName("Find by vorgangId")
	@Nested
	class TestFindByVorgangId {

		private final GrpcFindVorgangAttachedItemRequest request = GrpcFindVorgangAttachedItemRequestTestFactory.create();
		private final GrpcFindVorgangAttachedItemResponse response = GrpcFindVorgangAttachedItemResponseTestFactory.create();
		private final Kommentar kommentar = KommentarTestFactory.create();

		@Mock
		private VorgangAttachedItemServiceBlockingStub grpcServiceWithInterceptor;

		@BeforeEach
		void init() {
			doReturn(grpcServiceWithInterceptor).when(service).getGrpcServiceWithInterceptor();
			when(grpcServiceWithInterceptor.find(request)).thenReturn(response);
		}

		@Test
		void shouldGetGrpcServiceWithInterceptor() {
			callService();

			verify(service).getGrpcServiceWithInterceptor();
		}

		@Test
		void shouldCallGrpcService() {
			callService();

			verify(grpcServiceWithInterceptor).find(request);
		}

		@Test
		void shouldCallMapper() {
			callService().toList();

			verify(mapper).fromItem(GrpcFindVorgangAttachedItemResponseTestFactory.ATTACHED_ITEM);
		}

		@Test
		void shouldReturnFoundKommentare() {
			when(mapper.fromItem(GrpcFindVorgangAttachedItemResponseTestFactory.ATTACHED_ITEM)).thenReturn(kommentar);

			var foundKommentare = callService().toList();

			assertThat(foundKommentare).containsExactly(kommentar);
		}

		private Stream<Kommentar> callService() {
			return service.findByVorgangId(GrpcFindVorgangAttachedItemRequestTestFactory.VORGANG_ID);
		}
	}

	@Nested
	class TestGetGrpcServiceWithInterceptor {

		private MockedConstruction<OzgCloudCallContextAttachingInterceptor> mockedConstructionInterceptor;
		private OzgCloudCallContextProvider argumentContextProvider;
		@Mock
		private OzgCloudCallContextAttachingInterceptor interceptor;

		@BeforeEach
		void setUpMocks() {
			mockedConstructionInterceptor = mockConstruction(OzgCloudCallContextAttachingInterceptor.class, (mock, context) -> {
				argumentContextProvider = (OzgCloudCallContextProvider) context.arguments().get(0);
				interceptor = mock;
			});
		}

		@AfterEach
		void cleanUp() {
			mockedConstructionInterceptor.close();
		}

		@Test
		void shoudCreateOzgCloudCallContextAttachingInterceptor() {
			callService();

			assertThat(mockedConstructionInterceptor.constructed()).hasSize(1);
		}

		@Test
		void shoudCreateOzgCloudCallContextAttachingInterceptorWithContextProvider() {
			callService();

			assertThat(argumentContextProvider).isEqualTo(contextProvider);
		}

		@Test
		void shouldSetInterceptor() {
			callService();

			verify(grpcService).withInterceptors(interceptor);
		}

		@Test
		void shouldReturnServiceStub() {
			when(grpcService.withInterceptors(any(OzgCloudCallContextAttachingInterceptor.class))).thenReturn(grpcService);

			var returnedServiceStub = callService();

			assertThat(returnedServiceStub).isEqualTo(grpcService);
		}

		private VorgangAttachedItemServiceBlockingStub callService() {
			return service.getGrpcServiceWithInterceptor();
		}
	}

}
