package de.ozgcloud.archive.quittung;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.archive.common.command.ArchiveManagerCommand;
import de.ozgcloud.archive.common.command.CommandService;
import de.ozgcloud.archive.common.command.CommandTestFactory;
import de.ozgcloud.archive.common.xta.AbgabeImportBestaetigen0402TestFactory;
import de.ozgcloud.archive.common.xta.XtaFileHelper;
import de.ozgcloud.archive.common.xta.XtaFileParser;
import de.ozgcloud.archive.common.xta.XtaFileTestFactory;
import de.ozgcloud.archive.common.xta.XtaFileType;
import de.ozgcloud.archive.common.xta.XtaImportConfirmationHandler;
import de.ozgcloud.archive.common.xta.XtaImportConfirmationHandler.XtaAbgabeAdapter;
import de.ozgcloud.archive.common.xta.XtaMessageTestFactory;
import de.ozgcloud.archive.common.xta.XtaService;
import de.ozgcloud.archive.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.command.CommandExecutedEvent;
import de.ozgcloud.command.CommandFailedEvent;
import de.ozgcloud.xta.client.model.XtaFile;
import de.ozgcloud.xta.client.model.XtaMessage;

class QuittungServiceTest {

	@Spy
	@InjectMocks
	private QuittungService service;
	@Mock
	private XtaService xtaService;
	@Mock
	private CommandService commandService;
	@Mock
	private ApplicationEventPublisher eventPublisher;
	@Mock
	private XtaFileHelper xtaFileHelper;
	@Mock
	private XtaFileParser xtaFileParser;

	@Nested
	class TestFetchQuittung {

		@Captor
		private ArgumentCaptor<Consumer<XtaMessage>> consumerCaptor;

		private final XtaMessage message = XtaMessageTestFactory.create();

		@Test
		void shouldCallXtaService() {
			service.fetchQuittung();

			verify(xtaService).fetchMessages(any());
		}

		@Test
		void shouldPassQuittungConsumerToXtaService() {
			service.fetchQuittung();

			verify(xtaService).fetchMessages(consumerCaptor.capture());
			consumerCaptor.getValue().accept(message);
			verify(service).consumeQuittung(message);
		}
	}

	@DisplayName("Consume Quittung")
	@Nested
	class TestConsumeQuittung {

		@Mock
		private XtaImportConfirmationHandler xtaFileContentHandler;

		private final XtaMessage xtaMessage = XtaMessageTestFactory.create();

		@Test
		void shouldCallIsImportConfirmation() {
			service.consumeQuittung(xtaMessage);

			verify(service).isImportConfirmation(XtaMessageTestFactory.MESSAGE_FILE);
		}

		@DisplayName("on import confirmation")
		@Nested
		class TestOnImportConfirmation {

			@Mock
			private XtaAbgabeAdapter abgabeHandler;

			@BeforeEach
			void mock() {
				doReturn(true).when(service).isImportConfirmation(any());
				when(xtaFileHelper.initImportConfirmationHandler(any())).thenReturn(xtaFileContentHandler);
				when(xtaFileContentHandler.getAbgaben()).thenReturn(Stream.of(abgabeHandler, abgabeHandler));
				doNothing().when(service).handleAbgabe(any());
			}

			@Test
			void shouldInitContentHandler() {
				service.consumeQuittung(xtaMessage);

				verify(xtaFileHelper).initImportConfirmationHandler(XtaMessageTestFactory.MESSAGE_FILE);
			}

			@DisplayName("should handle import confirmation if type from document is matching")
			@Test
			void shouldCallHandleAbgabe() {
				service.consumeQuittung(xtaMessage);

				verify(service, times(2)).handleAbgabe(abgabeHandler);
			}
		}

		@DisplayName("should not handle message if its not an import confirmation")
		@Test
		void shouldNotProceed() {
			doReturn(false).when(service).isImportConfirmation(any());

			service.consumeQuittung(xtaMessage);

			verify(service, never()).handleAbgabe(any());
			verify(xtaFileHelper, never()).initImportConfirmationHandler(any());
		}
	}

	@Nested
	class TestIsImportConfirmation {

		private XtaFile xtaFile = XtaFileTestFactory.create();

		@Test
		void shouldCallXtaFileParser() {
			service.isImportConfirmation(xtaFile);

			verify(xtaFileParser).isType(xtaFile, XtaFileType.IMPORT_CONFIRMATION);
		}

		@ParameterizedTest
		@ValueSource(booleans = { true, false })
		void shouldReturnIsType(boolean isImportConfirmation) {
			when(xtaFileParser.isType(any(), any())).thenReturn(isImportConfirmation);

			var returnValue = service.isImportConfirmation(xtaFile);

			assertThat(returnValue).isEqualTo(isImportConfirmation);
		}

	}

	@DisplayName("Handle abgabe")
	@Nested
	class TestHandleAbgabe {

		@Mock
		private XtaAbgabeAdapter abgabeHandler;

		@BeforeEach
		void mock() {
			when(abgabeHandler.getVorgangId()).thenReturn(VorgangWithEingangTestFactory.ID);
			when(commandService.findPending(any(), any())).thenReturn(Stream.empty());
		}

		@Test
		void shouldGetVorgangId() {
			handleImportConfirmation();

			verify(abgabeHandler).getVorgangId();
		}

		@Test
		void shouldCallCommandService() {
			handleImportConfirmation();

			verify(commandService).findPending(VorgangWithEingangTestFactory.ID, QuittungService.ARCHIVE_VORGANG_ORDER);
		}

		@DisplayName("on existing pending commands")
		@Nested
		class TestOnExistingPendingCommands {

			private final ArchiveManagerCommand command = CommandTestFactory.create();
			private final List<ArchiveManagerCommand> commands = List.of(command);

			@BeforeEach
			void mock() {
				when(commandService.findPending(any(), any())).thenReturn(commands.stream());
				doNothing().when(service).evaluateAbgabe(any(), any());
			}

			@Test
			void shouldCallEvaluateConfirmation() {
				handleImportConfirmation();

				verify(service).evaluateAbgabe(abgabeHandler, commands);
			}
		}

		@Test
		void shouldNotCallEvaluateImportConfirmationOnEmptyList() {
			handleImportConfirmation();

			verify(service, never()).evaluateAbgabe(any(), any());
		}

		private void handleImportConfirmation() {
			service.handleAbgabe(abgabeHandler);
		}
	}

	@DisplayName("Evaluate abgabe")
	@Nested
	class TestEvaluateConfirmation {

		@Mock
		private XtaAbgabeAdapter abgabeHandler;

		private final ArchiveManagerCommand command = CommandTestFactory.create();
		private final List<ArchiveManagerCommand> commands = List.of(command, command);

		@Test
		void shouldCallIsSuccessfullyDone() {
			service.evaluateAbgabe(abgabeHandler, commands);

			verify(abgabeHandler).isSuccessful();
		}

		@Test
		void shouldCallPublishCommendExcecutedEvent() {
			when(abgabeHandler.isSuccessful()).thenReturn(true);

			service.evaluateAbgabe(abgabeHandler, commands);

			verify(service, times(2)).publishCommandExecutedEvent(command);
		}

		@Test
		void shouldCallPublishCommandFailedEvent() {
			when(abgabeHandler.isSuccessful()).thenReturn(false);

			service.evaluateAbgabe(abgabeHandler, commands);

			verify(service, times(2)).publishCommandFailedEvent(abgabeHandler, command);
		}

		@DisplayName("on successfully done import confirmation")
		@Nested
		class TestPublishCommandExcutedEvent {

			@Captor
			private ArgumentCaptor<CommandExecutedEvent> commandExecutedEventCaptor;

			@DisplayName("should publish command executed event")
			@Test
			void shouldPublishCommandExecutedEvent() {
				service.publishCommandExecutedEvent(command);

				verify(eventPublisher).publishEvent(commandExecutedEventCaptor.capture());
				assertThat(commandExecutedEventCaptor.getValue().getCommand()).isEqualTo(command);
			}
		}

		@DisplayName("on failed import confirmation")
		@Nested
		class TestPublishCommandFailedEvent {

			@Captor
			private ArgumentCaptor<CommandFailedEvent> commandFailedEventCaptor;

			private final ArchiveManagerCommand command = CommandTestFactory.create();

			@DisplayName("should publish command failed event")
			@Test
			void shouldPublishCommandFailedEvent() {
				when(abgabeHandler.getErrorMessage()).thenReturn(AbgabeImportBestaetigen0402TestFactory.FEHLERMELDUNG);

				service.publishCommandFailedEvent(abgabeHandler, command);

				verify(eventPublisher).publishEvent(commandFailedEventCaptor.capture());
				assertThat(commandFailedEventCaptor.getValue().getSource()).isEqualTo(CommandTestFactory.ID);
				assertThat(commandFailedEventCaptor.getValue().getErrorMessage()).isEqualTo(AbgabeImportBestaetigen0402TestFactory.FEHLERMELDUNG);
			}
		}
	}
}
